/** package name: jarnaby **/
package edu.byuh.cis.jarnaby;
/** imports **/
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.util.List;
import java.util.Map;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import android.annotation.TargetApi;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.res.AssetFileDescriptor;
import android.content.res.AssetManager;
import android.content.res.ColorStateList;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Rect;
import android.graphics.drawable.BitmapDrawable;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.Display;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.view.ViewGroup.LayoutParams;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ViewFlipper;

/** main class **/
public class Main extends Activity implements OnClickListener, OnTouchListener{
	/** attributes **/
	//tag for LogCat
	public final static String TAG = "jarnaby";

	//canvas for images [relative layout]
	private Canvas canvas;

	//attributes for UI
	private static Display display;
	private static ViewFlipper contents;
	private static TextView mainPathLabel;
	private static ImageButton aerialButton;
	private static ImageButton prevButton;
	private static ImageButton nextButton;
	private static ImageButton linkButton;
	private static Button storyText;
	private static ScrollView storyArea;
	private static boolean aerialSwitch = false;

	//buttons on UI
	private final static String PAGELINK = "pagelink"; //link button at the right top corner
	private final static String LINKBOX = "linkbox";   //any link box
	private final static String PAGEIMG = "pageimage"; //thumbnail image

	//storing data
	private Map<Integer, Path> paths;
	private Map<Integer, Page> pages;
	private Map<String, String> images;

	//story title
	private String appTitle;
	private String titleFile;

	//image for background
	private static BitmapDrawable background;

	//current page
	private static int currentPageIndex;
	private static int currentPathIndex;
	private static int currentPageInPath;

	//for on-touch listener
	private static int downXValue;
	private static int downYValue;
	private static int upXValue = 0;
	private static int upYValue = 0;

	//mapPage
	private static RelativeLayout mapPage;

	//audio
	private MediaPlayer player;
	private MediaPlayer soundPlayer;
	private String bkgdMusic;

	private static ViewFlipper flipper;
	RelativeLayout introPage;
	View beginButton;
	View instructionsButton;

	private Bundle creditsBundle;
	private Map<String, String> credits;
	
	private String storyDir;
	private boolean defaultStory;

	boolean mExternalStorageAvailable;
	boolean mExternalStorageWriteable;

	private static final int END_CHOICE = 0;

	private static final int STORY_SELECT = 1;
	
	private boolean home;

	//	private static final int INVISIBLE = 4;
	//	private static final int VISIBLE = 0;
	//
	//	private static final int BLACK = -16777216;

	private Toast toast;

	AssetManager assets;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		setVolumeControlStream(AudioManager.STREAM_MUSIC);

		flipper = (ViewFlipper) findViewById(R.id.flippy);
		introPage=(RelativeLayout) findViewById(R.id.intro);
		beginButton = (Button) findViewById(R.id.begin);
		instructionsButton = (Button) findViewById(R.id.instructions);

		beginButton.setOnClickListener(this);
		instructionsButton.setOnClickListener(this);

		toast = Toast.makeText(this, "Default", Toast.LENGTH_SHORT);
		toast.setGravity(Gravity.CENTER, 0, 0);

//		File folder = new File(Environment.getExternalStorageDirectory() + "/Jarnaby/stories/LRRH");
//		folder.mkdirs();
//		folder = new File(Environment.getExternalStorageDirectory() + "/Jarnaby/stories/LittleBoePeep");
//		folder.mkdirs();
//		folder = new File(Environment.getExternalStorageDirectory() + "/Jarnaby/stories/Goldy");
//		folder.mkdirs();

		File storyFolder = new File(Environment.getExternalStorageDirectory() + "/Jarnaby/stories");
//
//		File storyFolder = new File("/sdcard/Jarnaby/stories");
//		storyFolder.mkdirs();
//		
		String[] stories;
		stories = storyFolder.list();

//		if(stories != null){
//			Bundle storyBundle = new Bundle();
//			storyBundle.putStringArray("storyOptions", stories);
//			Intent i = new Intent(this, StoryList.class);
//			i.putExtras(storyBundle);
//			startActivityForResult(i, STORY_SELECT);
//		}

		setup();
		try {
			Log.d(TAG, "Entering bundle check...");
			if(this.getIntent().hasExtra("choice")){
				Log.d(TAG, "Inside bundle check...");
			Bundle tempBundle = new Bundle();
			tempBundle = this.getIntent().getExtras();
			storyDir = (String) tempBundle.get("choice");
			}
			else storyDir = "LRRH";
			Log.d(TAG, "Bundled Story Directory at: "+ storyDir);
			
			defaultStory = true;
			if(storyDir != null){
				Log.d(TAG, "StoryDir is: "+ storyDir);
			}
			else Log.d(TAG, "StoryDir is NULL!!!");
			
			if(storyDir.equals("LRRH")){
				assets = getAssets();
				loadState(assets.open(Config.getStoryPath() + "story.xml"));
			}
			else{
				defaultStory = false;
				for(String imgPath : images.values()){
					imgPath = "/sdcard/Jarnaby/stories/" + storyDir +"/images/";
				}
				Log.d(TAG, "loading file: "+ "/sdcard/Jarnaby/stories/"+ storyDir +"/story.xml");
				loadState(new FileInputStream("/sdcard/Jarnaby/stories/"+ storyDir +"/story.xml"));
			}
			
			home = true;
			player = new MediaPlayer();
			soundPlayer = new MediaPlayer();
			setTitle(appTitle);
			introPage.setBackgroundDrawable(setImage(titleFile));

			if (soundPlayer.isPlaying())
			{
				soundPlayer.stop();
			}

			RelativeLayout.LayoutParams lp = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.WRAP_CONTENT, RelativeLayout.LayoutParams.WRAP_CONTENT);
			lp.setMargins(getWindowWidth()/6, 0, getWindowWidth()/8, 10);
			lp.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
			beginButton.setLayoutParams(lp);

			if(Options.getMusic(this)){
				//				Music.play(this, afd);
				playMusic();
			}

			//loadState(new File(Config.getStoryPath() + Config.storyPrefix + ".xml"));
		} catch (SAXException e) {
			Log.e(TAG, "invalid XML!: " + e.toString());
		} catch (IOException e) {
			Log.e(TAG, "cannot find file.: " + e.toString());
		}
	}

	private void playMusic(){
		if (player.isPlaying())
		{
			player.stop();
		}
		if(bkgdMusic != null){
			AssetFileDescriptor afd;
			try {
				afd = getAssets().openFd(bkgdMusic);
				player.setDataSource(afd.getFileDescriptor(),afd.getStartOffset(),afd.getLength());
				player.prepare();
				player.setLooping(true);
				player.setVolume((float) 0.2, (float) 0.2);
				player.start();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private void playSound(){
		if(Options.getSfx(this)){
			String filePath = pages.get(currentPageIndex).getSoundPath();
			if(filePath != null){
				AssetFileDescriptor afd;
				try {
					if (soundPlayer != null) soundPlayer.release();
					afd = getAssets().openFd(filePath);
					soundPlayer = new MediaPlayer();
					soundPlayer.setDataSource(afd.getFileDescriptor(),afd.getStartOffset(),afd.getLength());
					soundPlayer.prepare();
					soundPlayer.setVolume((float) 0.9, (float) 0.9);
					soundPlayer.start();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	/** set up **/
	private void setup() {
		//get display information
		display = ((WindowManager) getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay();

		//set XML property and parse from XML. [Required]
		System.setProperty("org.xml.sax.driver", "org.xmlpull.v1.sax2.Driver");

		//find view
		contents = (ViewFlipper) findViewById(R.id.contents);
		mainPathLabel = (TextView) findViewById(R.id.mainPathLabel);
		aerialButton = (ImageButton) findViewById(R.id.aerialButton);
		prevButton = (ImageButton) findViewById(R.id.prevButton);
		nextButton = (ImageButton) findViewById(R.id.nextButton);
		linkButton = (ImageButton) findViewById(R.id.linkButton);
		storyText = (Button) findViewById(R.id.storyText);
		storyArea = (ScrollView) findViewById(R.id.storyArea);

		//add view to listener
		contents.setOnTouchListener(this);
		aerialButton.setOnClickListener(this);
		prevButton.setOnClickListener(this);
		nextButton.setOnClickListener(this);
		linkButton.setOnClickListener(this);
		storyArea.setOnTouchListener(this);

	}

	/** load information from XML **/
	private void loadState(InputStream inputStream) throws SAXException, IOException {
		//parse XML
		XMLReader xr = XMLReaderFactory.createXMLReader();
		StoryParser state = new StoryParser(getWindowWidth(), getWindowHeight());
		xr.setContentHandler(state);
		xr.setErrorHandler(state);
		//FileReader r = new FileReader(inputStream);
		xr.parse(new InputSource(inputStream));

		//pull data from the XML and store
		paths = state.getPaths();
		pages = state.getPages();
		images = state.getImages();

		appTitle = state.getTitle();
		titleFile = state.getTitleFile();

		bkgdMusic = state.getMusic();

		if(state.getCredits() != null){
			credits = state.getCredits();
			creditsBundle = new Bundle();
			if(creditsBundle != null){
				creditsBundle.putString("author", credits.get("author"));
				creditsBundle.putString("artwork", credits.get("artwork"));
				creditsBundle.putString("music", credits.get("music"));
				creditsBundle.putString("voice", credits.get("voice"));
				creditsBundle.putString("summary", credits.get("summary"));
			}
			else Log.d(Main.TAG, "bundle is NULL!!!"); 
		}
		else Log.d(Main.TAG, "credits returned as NULL!!!"); 

		//loop through set canvas for all pages
		for (int i = 0; i < pages.size(); i++) {
			contents.addView(new Canvas(this));
		}

		//loop through pages and set image path
		for (Map.Entry<Integer, Page> p : pages.entrySet()) {
			int page_id = p.getKey();
			Page page = p.getValue();
			try {
				page.setPageImagePath(images.get(page.pictID));
			} catch( Error e ) {
				Log.w(TAG, e.toString());
			}
			canvas = new Canvas(this);

			mainPathLabel.setTextColor(Color.BLACK);
			mainPathLabel.setGravity(Gravity.CENTER);
			linkButton.setVisibility(View.INVISIBLE);
			mainPathLabel.setVisibility(View.INVISIBLE); 

			if (page.hasLink()) {
				for (Link l :page.getLinks()) {
					int path_id;
					Path path = null;
					int backPID = 0;
					findpath: for (Map.Entry<Integer, Path> pth : paths.entrySet()) {
						path_id = pth.getKey();
						path = pth.getValue();
						for (Integer pid : path.placeholders){
							if(page.getID() == pid){
								backPID = pid;
								break findpath;
							}
						}
					}
					if(path != null){
						//Page tester = pages.get(l.toPageIndex);
						Link testy = new Link(path, page.getID());
						testy.toPageID = backPID;
						//						if(tester == null) Log.d(TAG, "Null Page!!!");
						//						else {
						//							Log.d(TAG, "Adding link to "+ tester.getPictID());
						//							tester.setBacklink(testy);
						//						}
						//						Log.d(TAG, "7");
						//						pages.get(l.toPageIndex).setBacklink(new Link(path, page.getID()));
						//						Log.d(TAG, "8");
					}
				}
			}
			/*else{
				//linkButton.setVisibility(INVISIBLE); //troubleshooting
			}*/
			//			if (page.hasLink()) {
			//				for (Link l :page.getLinks()) {
			//					int path_id;
			//					Path path = null;
			//					int backPID = 0;
			//					Log.d(TAG, "fromPageID: "+ page.getID() +" pageID: "+ l.toPageID + " pageIndex: "+ l.toPageIndex);
			//					if(l.toPageIndex == 0){
			//						findpath: for (Map.Entry<Integer, Path> pth : paths.entrySet()) {
			//							path_id = pth.getKey();
			//							path = pth.getValue();
			//							for (Integer pid : path.placeholders){
			//								if(page.getID() == pid){
			//									backPID = pid;
			//									break findpath;
			//								}
			//							}
			//						}
			//						if(path != null){
			//							Page tester = l.toPath.getPages().get(0);
			//							Link testy = new Link(path, page.getID());
			//							testy.toPageID = backPID;
			//							if(tester == null) Log.d(TAG, "Null Page!!!");
			//							else {
			//								Log.d(TAG, "Adding link to "+ tester.getPictID());
			//								tester.setBacklink(testy);
			//							}
			//							Log.d(TAG, "7");
			////							Log.d(TAG, "8");
			//						}
			//					}
			////					findpath: for (Map.Entry<Integer, Path> pth : paths.entrySet()) {
			////						path_id = pth.getKey();
			////						path = pth.getValue();
			////						for (Integer pid : path.placeholders){
			////							if(l.getID() == pid){
			////								backPID = pid;
			////								break findpath;
			////							}
			////						}
			////					}
			//					if(path != null){
			//						Page tester = pages.get(l.toPageIndex);
			//						Link testy = new Link(path, page.getID());
			//						testy.toPageID = backPID;
			//						if(tester == null) Log.d(TAG, "Null Page!!!");
			//						else {
			//							Log.d(TAG, "Adding link to "+ tester.getPictID());
			//							tester.setBacklink(testy);
			//						}
			//						Log.d(TAG, "7");
			////						Log.d(TAG, "8");
			//					}
			//				}
			//			}
			contents.addView(canvas, page_id);
			clean(canvas);
			clean(page);
		}

		currentPathIndex = 0;
		currentPageIndex = 0;
		currentPageInPath = 0;
		pages.get(currentPageIndex).setVisited(true);
		if(currentPageIndex == 0){
			prevButton.setVisibility(View.INVISIBLE);
			if(pages.get(0).hasLink()){
				linkButton.setVisibility(View.VISIBLE);
			}
		}

		//add one more view for aerial view 
		mapPage = new Canvas(this);	
		//mapPage.setBackgroundDrawable(getResources().getDrawable(R.drawable.backrepeat));
		mapPage.setBackgroundColor(Color.WHITE);
		//String uri = "drawable/bkgd_texture";
		//int imageResource = getResources().getIdentifier(uri, null, getPackageName());
		//mapPage.setBackgroundDrawable(getResources().getDrawable(imageResource));
		int mapPageWidth = LayoutParams.FILL_PARENT;
		int mapPageHeight = Config.thumbnailHeight * 2 * contents.getChildCount();
		mapPage.setLayoutParams(new RelativeLayout.LayoutParams(mapPageWidth, mapPageHeight));
		contents.addView(mapPage);
		reconstruct();

		contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(paths.get(currentPathIndex).placeholders.get(currentPageInPath)).getPageImage()));
		contents.setDisplayedChild(currentPageIndex);
		addText(); 
		//		playSound();  //plays at title screen >.<
	}

	@Override
	protected void onPause(){
		super.onPause();
		//		Music.pause(this);
		if(player != null) player.pause();
		if(soundPlayer != null) soundPlayer.pause();
	}

	@Override
	protected void onResume(){
		super.onResume();
		if(Options.getMusic(this)){
			if(player != null) player.start();
			else playMusic();
		}
		if(Options.getSfx(this)){
			if(soundPlayer != null) soundPlayer.start();
//			else playMusic();
		}
	}

	/** on stop **/
	@Override
	protected void onStop() {
		super.onStop();
		//System.exit(0);
	}

	private static int prevX;
	@Override
	public boolean onTouch(View v, MotionEvent event) {
		int x = (int) event.getX();
		int y = (int) event.getY();

		if(v == storyArea || v == storyText) return false;
		else {
			switch(event.getAction()) {
			case MotionEvent.ACTION_DOWN:
				//				touchX = x;
				//				touchY = y;

				downXValue = x - upXValue;
				downYValue = y - upYValue;
				prevX = x;

				if( contents.getCurrentView() == contents.getChildAt(contents.getChildCount()-1)) {
					mapPage = (RelativeLayout) contents.getChildAt(contents.getChildCount()-1);
				}
				break; 
			case MotionEvent.ACTION_UP:
				upXValue = x - downXValue; 
				upYValue = y - downYValue;

				if(!aerialSwitch) {

					if((prevX < x) && ((x - prevX > 10))) prevPage();
					else if((prevX > x) && ((prevX - x > 10)))nextPage();
					else{
						Page page = pages.get(currentPageIndex);
						if (page.hasLink()) {
							for (Link l :page.getLinks()) {
								//TODO check if link path is equal to current path
								if(l.rect != null && l.toPath.getID() != currentPathIndex){
									Rect r = new Rect(l.rect.left, l.rect.top,
											l.rect.left+l.rect.right, l.rect.top+l.rect.bottom);
									if(r.contains(x, y)){
										goToLink(l.toPageID, l.toPath.getID());
										return true;
									}
								}
							}
						}
					}
				}
				break;

			case MotionEvent.ACTION_MOVE:
				if (aerialSwitch) {
					if( contents.getCurrentView() == mapPage) {
						if(downXValue > getWindowWidth()) downXValue = getWindowWidth();
						if(downXValue < 0) downXValue = 0;
						//						Log.d(TAG, "touch scrolling to: "+ (downXValue - x) + ", "+ (downYValue - y));

						mapPage.scrollTo( downXValue - x, downYValue - y);
					}
				}
			}
			return true;
		}
	}

	/** on-click **/
	@Override
	public void onClick(View v) {

		switch(v.getId()){
		case R.id.begin:
			flipper.setInAnimation(Animations.inFromRightAnimation());
			flipper.setOutAnimation(Animations.outToLeftAnimation());
			flipper.showNext();
			home = false;
			playSound();
			break;
		case R.id.instructions:
//			Intent i = new Intent(this, Instructions.class);
//			startActivity(i);
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setTitle("Instructions");
			builder.setMessage(R.string.instructions_text);
			builder.setNeutralButton("Done", null);
			builder.create().show();
			break;
		}

		if (v == prevButton) prevPage();

		if (v == nextButton) nextPage();

		if (v == linkButton){
			Page page = pages.get(currentPageIndex);
			Link l = page.getLinks().get(0);
			goToLink(l.toPageID, l.toPath.getID());
		}

		if (v == aerialButton) {
			aerialSwitch = !aerialSwitch;
			if (aerialSwitch) {
				mainPathLabel.setText("Currently viewing \n"+ paths.get(currentPathIndex).getName() +"'s perspective");
				mainPathLabel.setVisibility(View.VISIBLE); 
//				contents.setInAnimation(Animations.inFromTopAnimation());
//				contents.setOutAnimation(Animations.outToBottomAnimation());
				contents.setDisplayedChild(contents.getChildCount() - 1);
			} else {
				mainPathLabel.setVisibility(View.INVISIBLE); 
//				contents.setInAnimation(Animations.inFromBottomAnimation());
//				contents.setOutAnimation(Animations.outToTopAnimation());
				contents.setDisplayedChild(currentPageIndex);
				pages.get(currentPageIndex).setVisited(true);
			}
			contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(currentPageIndex).getPageImage()));
		}
		reconstruct();
	}

	@Override
	public void onBackPressed(){
		//prevPage();
		if (aerialSwitch)
		{
			((Canvas)mapPage).resetLines();
			mapPage.removeAllViews();
			aerialSwitch = false;
			mainPathLabel.setVisibility(View.INVISIBLE);
			nextButton.setVisibility(View.VISIBLE);
			aerialButton.setVisibility(View.VISIBLE);
			if (currentPageIndex != 0)
			{
				prevButton.setVisibility(View.VISIBLE);
			}
			storyArea.setVisibility(View.VISIBLE);
			if(pages.get(currentPageIndex).hasLink()){
				if(pages.get(currentPageIndex).getLinks().get(0).toPath.getID() != currentPathIndex){
					linkButton.setVisibility(View.VISIBLE);
				}
			}
			setAnimation();

			contents.getChildAt(currentPageIndex).setBackgroundDrawable(null);
			
			for( int i = 0; i < paths.get(currentPathIndex).placeholders.size(); i++ ) {
				if(paths.get(currentPathIndex).placeholders.get(i) == currentPageIndex) {
					currentPageInPath = i;
					break;
				}
			}
			if(!aerialSwitch) {
				contents.setDisplayedChild(currentPageIndex);
				contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(currentPageIndex).getPageImage()));
			}
			addText();
			playSound();
			pages.get(currentPageIndex).setVisited(true);
		}
		else if (home)
		{
			System.exit(0);
		}
		else
		{
			onCreate(creditsBundle);
		}
		return;
	}

	public void prevPage() {
//		contents.setInAnimation(Animations.inFromLeftAnimation());
//		contents.setOutAnimation(Animations.outToRightAnimation());
		if(currentPageIndex == 1){
			prevButton.setVisibility(View.INVISIBLE);
		}
		//move backward if the page is not the first page of pages in the path
		if (paths.get(currentPathIndex).placeholders.get(currentPageInPath) != paths.get(currentPathIndex).placeholders.get(0)) {
			currentPageInPath--;
			currentPageIndex = paths.get(currentPathIndex).placeholders.get(currentPageInPath);
			if(!aerialSwitch) {
				contents.setDisplayedChild(currentPageIndex);
				if(currentPageInPath < paths.get(currentPathIndex).placeholders.size() - 2 ) contents.getChildAt(currentPageInPath + 2).setBackgroundDrawable(null);
				contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(currentPageIndex).getPageImage()));
			}
			addText();
			pages.get(currentPageIndex).setVisited(true);
			playSound();
		} else {
			//link back
			if(currentPageInPath == 0){
				boolean found = false;
				for (Map.Entry<Integer, Page> page : pages.entrySet()) {
					if (page.getValue().hasLink()) {
						for (Link l :page.getValue().getLinks()) {
							if ( l.toPageID == currentPageIndex && l.toPath.getID() == currentPathIndex ) {
								currentPageIndex = page.getKey();
								for( int i: paths.keySet()) {
									if ( i != currentPathIndex && paths.get(i).placeholders.contains(currentPageIndex)) {
										currentPathIndex = i;
										for(int j = 0; j < paths.get(i).placeholders.size(); j++ ) {
											if(paths.get(i).placeholders.get(j) == currentPageIndex){
												currentPageInPath = j;
												if(!aerialSwitch) {
													contents.setDisplayedChild(currentPageIndex);
													if(currentPageInPath < paths.get(currentPathIndex).placeholders.size() - 2 ) contents.getChildAt(currentPageInPath + 2).setBackgroundDrawable(null);
													contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(currentPageIndex).getPageImage()));
												}
												addText();
												toast.setText("Switching to "+ paths.get(currentPathIndex).getName() +"'s perspective");
												toast.show();
												pages.get(currentPageIndex).setVisited(true);
												playSound();
												found = true;
												break;
											}
										}
									}
									if (found) break;
								}
								if(found) break;
							}
						}
					}
					if (found) break;
				}
			}
		}
		if(pages.get(currentPageIndex).hasLink()){
			if(pages.get(currentPageIndex).getLinks().get(0).toPath.getID() != currentPathIndex){
				linkButton.setVisibility(View.VISIBLE);
			}
		}
		else{
			linkButton.setVisibility(View.INVISIBLE);
		}
	}

	public void nextPage() {
//		contents.setInAnimation(Animations.inFromRightAnimation());
//		contents.setOutAnimation(Animations.outToLeftAnimation());
		prevButton.setVisibility(View.VISIBLE);
		//move forward if the page is not the last page of pages in the path.
		if (paths.get(currentPathIndex).placeholders.get(currentPageInPath) != paths.get(currentPathIndex).placeholders.get(paths.get(currentPathIndex).placeholders.size() - 1)) {
			currentPageInPath++;
			currentPageIndex = paths.get(currentPathIndex).placeholders.get(currentPageInPath);
			if(!aerialSwitch) {
				contents.setDisplayedChild(currentPageIndex);
				if(currentPageInPath > 1) contents.getChildAt(currentPageInPath - 2).setBackgroundDrawable(null);
				contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(currentPageIndex).getPageImage()));
				//				String filePath = pages.get(currentPageIndex).getSoundPath();
				//				if(filePath != null){
				//					//playSound(filePath);
				//				}
				playSound();
			}
			addText();
			pages.get(currentPageIndex).setVisited(true);
			Log.d(TAG, "Image: " + background.toString());
		}
		else {
			if(!aerialSwitch){
//				Intent i = new Intent(this, EndPage.class);
//				startActivityForResult(i, END_CHOICE);
				AlertDialog.Builder builder = new AlertDialog.Builder(this);
				builder.setTitle("End of Story");
				builder.setMessage(R.string.end_page);
				builder.setPositiveButton("Map", new DialogInterface.OnClickListener()
				{
					@Override
					public void onClick(DialogInterface dialog, int which) {
						// TODO Auto-generated method stub
						aerialSwitch = !aerialSwitch;
						if (aerialSwitch) {
							mainPathLabel.setText("Currently viewing \n"+ paths.get(currentPathIndex).getName() +"'s perspective");
							mainPathLabel.setVisibility(View.VISIBLE); 
//							contents.setInAnimation(Animations.inFromTopAnimation());
//							contents.setOutAnimation(Animations.outToBottomAnimation());
							contents.setDisplayedChild(contents.getChildCount() - 1);
						} else {
							mainPathLabel.setVisibility(View.INVISIBLE); 
//							contents.setInAnimation(Animations.inFromBottomAnimation());
//							contents.setOutAnimation(Animations.outToTopAnimation());
							contents.setDisplayedChild(currentPageIndex);
							pages.get(currentPageIndex).setVisited(true);
						}
						contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(currentPageIndex).getPageImage()));
						reconstruct();
					}
					
				});
				builder.setNegativeButton("Home", new DialogInterface.OnClickListener() {
					
					@Override
					public void onClick(DialogInterface dialog, int which) {
						// TODO Auto-generated method stub
						onCreate(creditsBundle);
					}
				});
				builder.create();
				builder.show();
			}
		}
		if(pages.get(currentPageIndex).hasLink()){
			if(pages.get(currentPageIndex).getLinks().get(0).toPath.getID() != currentPathIndex){
				linkButton.setVisibility(View.VISIBLE);
			}
		}
		else{
			linkButton.setVisibility(View.INVISIBLE);
		}
	}

	protected void onActivityResult(int requestCode, int resultCode, Intent data){
		int choice;
		switch(requestCode){
		case END_CHOICE:
			switch(resultCode){
			case -1:
				if (resultCode == RESULT_CANCELED){
					//do something? or not?
				}
				else {
					Bundle tempBundle = new Bundle();
					tempBundle = data.getExtras();
					choice = tempBundle.getInt("choice");
					if(choice == 1){
						restart();
					}
					else if(choice == 0){
						aerialSwitch = !aerialSwitch;
//						contents.setInAnimation(Animations.inFromTopAnimation());
//						contents.setOutAnimation(Animations.outToBottomAnimation());
						contents.setDisplayedChild(contents.getChildCount() - 1);
						contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(currentPageIndex).getPageImage()));
						reconstruct();
					}
				}
				break;
			default:
				Log.d(TAG, "FAILURE RETURNING ANYTHING!!");
			}
			break;
		case STORY_SELECT:
			if(resultCode == RESULT_CANCELED){
				//do something
			}
			else{
				Bundle tempBundle = new Bundle();
				tempBundle = data.getExtras();
				storyDir = (String) tempBundle.get("choice");
				Log.d(TAG, "Story Directory at: "+ storyDir);
			}
		}
	}

	private void restart(){
		if(aerialSwitch){
			aerialSwitch = !aerialSwitch;
			mainPathLabel.setVisibility(View.INVISIBLE);
		}
		currentPathIndex = 0;
		currentPageInPath = 0;
		currentPageIndex = 0;
		contents.setDisplayedChild(currentPageIndex);
		if(currentPageInPath > 1) contents.getChildAt(currentPageInPath - 2).setBackgroundDrawable(null);
		contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(currentPageIndex).getPageImage()));
		addText();
		playSound();
	}

	/** reconstruct panel view **/
	private void reconstruct(){
		
		if (aerialSwitch) {
			mainPathLabel.setVisibility(View.VISIBLE); 
			nextButton.setVisibility(View.INVISIBLE);
			prevButton.setVisibility(View.INVISIBLE);
			linkButton.setVisibility(View.INVISIBLE);
			aerialButton.setVisibility(View.INVISIBLE);
			storyArea.setVisibility(View.INVISIBLE);
			if (soundPlayer != null && soundPlayer.isPlaying())
			{
				soundPlayer.stop();
			}
			//canvasMap = new Canvas(this);
//			((Canvas)mapPage).resetLines();
//			mapPage.removeAllViews();
		} else {
			mainPathLabel.setVisibility(View.INVISIBLE); 
		}

		//delete all the panels
		//		if(mapPage.getChildCount() > 0) {
		//			mapPage.removeAllViews();
		//			((Canvas)mapPage).resetLines();
		//		}

		if(aerialSwitch){
			//center screen on current node
			mapPage.scrollTo(0, currentPageInPath * ((int)(Config.thumbnailHeight+Config.thumbnailHeight*0.5))  - getWindowHeight()/2);
			upXValue = 0;
			upYValue = -currentPageInPath * ((int)(Config.thumbnailHeight+Config.thumbnailHeight*0.5)) + getWindowHeight()/2;
			downYValue = currentPageInPath * ((int)(Config.thumbnailHeight+Config.thumbnailHeight*0.5)) - getWindowHeight()/2;

			//reconstruct panels
			List<Integer> currentPath = paths.get(currentPathIndex).placeholders;
			for(int i = 0; i < currentPath.size() - 1; i++) {
				if (pages.get(currentPath.get(i)).isVisited()) {
					double space = 1.3;
					int x = ( getWindowWidth() - Config.thumbnailWidth ) / 2 +13;
					int y = (int)(i * Config.thumbnailHeight * space);
					int x2 = x + Config.thumbnailWidth + 10;
					int y2 = (int)((i+1) * Config.thumbnailHeight * space);
					int x3 = x - Config.thumbnailWidth - 10;
					int y3 = (int)((i-1) * Config.thumbnailHeight * space);
					int half = getWindowWidth()/2 + 1;

					mapPage.addView(addLink(new Rect(x, y, Config.thumbnailWidth, Config.thumbnailHeight), PAGEIMG, currentPath.get(i), currentPathIndex));
					((Canvas) mapPage).addLines(new Rect(half, y+Config.thumbnailWidth-36, half, y+Config.thumbnailHeight+(int)(6+Config.thumbnailHeight*space/2)));
					//canvasMap.addLines(new Rect(half, y+Config.thumbnailWidth+6, half, y+Config.thumbnailHeight+(int)(6+Config.thumbnailHeight*space/2)));
					mapPage.addView(addLink(new Rect(x, y2, Config.thumbnailWidth, Config.thumbnailHeight), PAGEIMG, currentPath.get(i+1), currentPathIndex));
					if(i != 0){
						((Canvas) mapPage).addLines(new Rect(half, y-Config.thumbnailWidth-6, half, y+Config.thumbnailHeight+(int)(6+Config.thumbnailHeight*space/2)));
						//canvasMap.addLines(new Rect(half, y-Config.thumbnailWidth-6, half, y+Config.thumbnailHeight+(int)(6+Config.thumbnailHeight*space/2)));
						mapPage.addView(addLink(new Rect(x, y3, Config.thumbnailWidth, Config.thumbnailHeight), PAGEIMG, currentPath.get(i-1), currentPathIndex));
					}
					if( pages.get(currentPath.get(i)).hasLink() ) {
						for( Link l: pages.get(currentPath.get(i)).getLinks() ) {
							if(l.toPath.getID() != currentPathIndex){								
								mapPage.addView(addLink(new Rect( x2-13, y2, Config.thumbnailWidth, Config.thumbnailHeight), PAGEIMG, l.toPageID, l.toPath.getID()));
								((Canvas) mapPage).addLines(new Rect( half, y+Config.thumbnailWidth-39, half+Config.thumbnailWidth+10, y+Config.thumbnailHeight+(int)(6+Config.thumbnailHeight*space/2)));
								//canvasMap.addLines(new Rect( half, y+Config.thumbnailWidth-32, half+Config.thumbnailWidth+10, y+Config.thumbnailHeight+(int)(6+Config.thumbnailHeight*space/2)));
								TextView pathLabel = new TextView(this);
								RelativeLayout.LayoutParams layoutParams = new RelativeLayout.LayoutParams(Config.thumbnailWidth, 50);
								pathLabel.setText("Switch to:\n"+l.toPath.getName());
								layoutParams.setMargins(x2-(Config.thumbnailHeight/6), y2+(Config.thumbnailHeight/4), 0, 0);
								pathLabel.setTextColor(Color.BLACK);
								pathLabel.setGravity(Gravity.CENTER);
								pathLabel.setTextSize(10);
								//pathLabel.setBackgroundDrawable(getResources().getDrawable(R.drawable.label_path));
								pathLabel.setLayoutParams(layoutParams);
								mapPage.addView(pathLabel);
							}
						}
					}
					//					if( pages.get(currentPath.get(i)).hasBacklink()){
					//						Log.d(TAG, "I HAVE A BACK LINK");
					//						Link l = pages.get(currentPath.get(i)).getBacklink();
					//						mapPage.addView(addLink(new Rect( x3, y3, Config.thumbnailWidth, Config.thumbnailHeight), PAGEIMG, l.toPageID, l.toPath.getID()));
					//						((Canvas) mapPage).addLines(new Rect( half, y+Config.thumbnailWidth+6, half+Config.thumbnailWidth+10, y+Config.thumbnailHeight+(int)(6+Config.thumbnailHeight*space/2)));
					//					}
				}
				
			}
			//mapPage.addView(canvasMap);
			clean(currentPath);
			//clean(canvasMap);
		}
	}

	/** 
	 * @param
	 * @return
	 */
	private RelativeLayout addLink(Rect r, String linkType, final int pageID, final int pathID) {
		int x = r.left;
		int y = r.top;
		int width = r.right;
		int height = r.bottom;
		RelativeLayout linkLayout = new RelativeLayout(this);
		RelativeLayout.LayoutParams layoutParams = null;
		RelativeLayout.LayoutParams notVisited = null;
		boolean whichLayout = true;
		if ( !linkType.equals(LINKBOX)) {
				layoutParams = new RelativeLayout.LayoutParams(Config.thumbnailWidth - 30, Config.thumbnailHeight - 30);
		} else {
			layoutParams = new RelativeLayout.LayoutParams(width, height);
		}
		layoutParams.setMargins(x, y, 0, 0);
		ImageButton linksButton = new ImageButton(this);

		//page link that displays at the top right corner
		if (linkType.equals(PAGELINK)) {
			linksButton.setImageDrawable(getResources().getDrawable(R.drawable.link));
			linksButton.setBackgroundColor(Color.TRANSPARENT);

			//set thumbnail links
		} else if(linkType.equals(PAGEIMG)){
			boolean visited = pages.get(pageID).isVisited();
			linksButton.setImageDrawable(resizeImage(setImage(pages.get(pageID).getPageImage()), visited));
			linksButton.setPadding(6, 6, 8, 8);
			if(pageID == currentPageIndex && pathID == currentPathIndex){
				linksButton.setBackgroundColor(Color.YELLOW);
			} else {
				linksButton.setBackgroundColor(Color.TRANSPARENT);
			}
			linksButton.setMaxWidth(width);
			linksButton.setMaxHeight(height);
			if (!visited)
			{
				linksButton.setEnabled(false);
				linksButton.setBackgroundColor(Color.DKGRAY);
				notVisited = new RelativeLayout.LayoutParams(Config.thumbnailWidth - 45, Config.thumbnailHeight - 45);
				notVisited.setMargins(x+7, y+7, 0, 0);
			}
			whichLayout = visited;
		} else if(linkType.equals(LINKBOX)){		
			linksButton.setBackgroundColor(Color.argb(50, 200, 0, 0));
		}
		//add action to image buttons
		linksButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				((Canvas)mapPage).resetLines();
				mapPage.removeAllViews();
				aerialSwitch = false;
				mainPathLabel.setVisibility(View.INVISIBLE);
				nextButton.setVisibility(View.VISIBLE);
				aerialButton.setVisibility(View.VISIBLE);
				if (pageID != 0)
				{
					prevButton.setVisibility(View.VISIBLE);
				}
				storyArea.setVisibility(View.VISIBLE);
				if(pages.get(pageID).hasLink()){
					if(pages.get(pageID).getLinks().get(0).toPath.getID() != pathID){
						linkButton.setVisibility(View.VISIBLE);
					}
				}
				setAnimation();

				if(currentPathIndex != pathID){
					toast.setText("Switching to "+ paths.get(pathID).getName() +"'s perspective");
					toast.show();
				}

				contents.getChildAt(currentPageIndex).setBackgroundDrawable(null);
				currentPageIndex = pageID;
				currentPathIndex = pathID;
				for( int i = 0; i < paths.get(currentPathIndex).placeholders.size(); i++ ) {
					if(paths.get(currentPathIndex).placeholders.get(i) == currentPageIndex) {
						currentPageInPath = i;
						break;
					}
				}
				if(!aerialSwitch) {
					contents.setDisplayedChild(currentPageIndex);
					contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(currentPageIndex).getPageImage()));
				}
				addText();
				playSound();
				pages.get(currentPageIndex).setVisited(true);
			}
		});
		if (whichLayout)
		{
			linksButton.setLayoutParams(layoutParams);
		}
		else
		{
			linksButton.setLayoutParams(notVisited);
		}
		linkLayout.addView(linksButton);
		clean(linksButton);
		clean(layoutParams);
		clean(notVisited);
		return linkLayout;
	}

	private void goToLink(final int pageID, final int pathID) {
		aerialSwitch = false;
		setAnimation();

		toast.setText("Switching to "+ paths.get(pathID).getName() +"'s perspective");
		toast.show();

		contents.getChildAt(currentPageIndex).setBackgroundDrawable(null);
		currentPageIndex = pageID;
		currentPathIndex = pathID;
		for( int i = 0; i < paths.get(currentPathIndex).placeholders.size(); i++ ) {
			if(paths.get(currentPathIndex).placeholders.get(i) == currentPageIndex) {
				currentPageInPath = i;
				break;
			}
		}
		if(!aerialSwitch) {
			contents.setDisplayedChild(currentPageIndex);
			contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(currentPageIndex).getPageImage()));
		}
		addText();
		playSound();
		pages.get(currentPageIndex).setVisited(true);
		if(pages.get(currentPageIndex).hasLink()){
			if(pages.get(currentPageIndex).getLinks().get(0).toPath.getID() != currentPathIndex){
				linkButton.setVisibility(View.VISIBLE);
			}
		}
		else{
			linkButton.setVisibility(View.INVISIBLE);
		}
	}


	//fade in and out
	public void setAnimation() {
//		contents.setInAnimation(this, android.R.anim.fade_in);
//		contents.setOutAnimation(this, android.R.anim.fade_out);	
	}

	/**
	 * add text to each page in storyArea
	 * it converts new lines and tabs
	 */
	public void addText(){
		boolean firstLine = true;
		String story = "";
		for (String s: pages.get(currentPageIndex).getStoryText().texts){
			if(!firstLine) story += "...\n\n";
			else firstLine = false;
			story += s.replace("\t", "").replace("\n", " ");
		}
		storyText.setText(story);
		clean(story);
	}

	/**
	 * setImage method which sets an image to the page
	 * @param fileName ... image file name in each page
	 * @return ... image file for each page
	 */
	public BitmapDrawable setImage(String fileName) {
		clean(background);
		try {
			InputStream imageFile = assets.open(fileName);
			if(!defaultStory){
				Log.d(TAG, "TRYING TO LOAD IMAGE");
				imageFile = new FileInputStream(fileName);
				
			}
			background = new BitmapDrawable(imageFile);
			clean(imageFile);
		} catch (IOException e) { Log.w(TAG, e.toString()); }
		if (background == null) background = (BitmapDrawable) getResources().getDrawable(R.drawable.image_not_found);
		//else background.setGravity(Gravity.TOP);
		return background;
	}

	/**
	 * @param  originalImage 		... original BitmapDrawable image 
	 * @param  visited       		... check to see if the page is visited
	 * @return BitmapDrawable Object... new image that is resized for thumbnail
	 */
	public BitmapDrawable resizeImage(BitmapDrawable originalImage, boolean visited){
		Bitmap bm = (originalImage).getBitmap();
		clean(originalImage);
		Bitmap resizedBitmap = null;
		try{
			int width = bm.getWidth();
			int height = bm.getHeight();
			float scaleWidth = ((float) Config.thumbnailWidth) / width;
			float scaleHeight = ((float) Config.thumbnailHeight) / height;
			Matrix matrix = new Matrix();
			matrix.postScale(scaleWidth, scaleHeight);
			resizedBitmap = Bitmap.createBitmap(bm, 0, 0, width, height, matrix, true);
			clean(matrix);
		} catch (Exception e) {
			resizedBitmap = bm;
			Log.e(TAG, "Error of Bitmap:" + e.toString());
		}
		BitmapDrawable d = new BitmapDrawable(resizedBitmap);
		clean(resizedBitmap);
		if (!visited)
		{
			d.setAlpha(100);
		}
		return d;
	}

	/** get width of the device screen **/
	public static int getWindowWidth() {
		return display.getWidth();
	}

	/** get height of the device screen **/
	public static int getWindowHeight() {
		return display.getHeight();
	}

	public static boolean getAerialSwitch(){
		return aerialSwitch;
	}

	public static void clean(Object o) {
		o = null;
	}

	@TargetApi(8)
	void createExternalStoragePrivateFile() {
		// Create a path where we will place our private file on external
		// storage.
		File file = new File(getExternalFilesDir(null), "DemoFile.jpg");
		File storyDir = new File("/sdcard/testermoot/");
		storyDir.mkdir();

		try {
			// Very simple code to copy a picture from the application's
			// resource into the external file.  Note that this code does
			// no error checking, and assumes the picture is small (does not
			// try to copy it in chunks).  Note that if external storage is
			// not currently mounted this will silently fail.
			InputStream is = getResources().openRawResource(R.drawable.splash);
			OutputStream os = new FileOutputStream(file);
			OutputStream obOs = new ObjectOutputStream(os); //maybe even try FileWriter
			byte[] data = new byte[is.available()];
			is.read(data);
			obOs.write(data);
			is.close();
			os.close();
			Log.d("ExternalStorage", "SUCCESSFUL writing: " + file);
		} catch (IOException e) {
			// Unable to create file, likely because external storage is
			// not currently mounted.
			Log.w("ExternalStorage", "Error writing " + file, e);
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu){
		super.onCreateOptionsMenu(menu);
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.menu, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		switch (item.getItemId()){
		case R.id.about:
			//startActivity(new Intent(this, About.class));
			builder.setTitle("About");
			builder.setMessage(R.string.about_text);
			builder.setNeutralButton("Done", null);
			builder.create();
			builder.show();
			return true;
		case R.id.credits:
//			Intent credits = new Intent(this, Credits.class);
//			credits.putExtras(creditsBundle);
//			startActivity(credits);
			String author = creditsBundle.getString("author");
			String artwork = creditsBundle.getString("artwork");
			String music = creditsBundle.getString("music");
			String voice = creditsBundle.getString("voice");
			String summary = creditsBundle.getString("summary");
			String credit;
			
			credit = "Author: "+ author +"\n\n"+ "Artwork: "+ artwork +"\n\n"+ 
			"Music: "+ music +"\n\n"+ "Voice: "+ voice +"\n\n"+ "Summary: "+ summary;
			
			builder.setTitle("Credits");
			builder.setMessage(credit);
			builder.setNeutralButton("Done", null);
			builder.create();
			builder.show();
			return true;
		case R.id.restart:
			restart();
			return true;
		case R.id.options:
			startActivity(new Intent(this, Options.class));
			return true;
		case R.id.exit:
			this.finish();
			return true;
			//			More items go here (if any) ...
		}
		return false;

	}

	@Override public boolean onKeyDown(int keyCode, KeyEvent event) { 
		switch (keyCode) { 
		case KeyEvent.KEYCODE_DPAD_CENTER: 
			Log.v("hey","pressed " + KeyEvent.KEYCODE_DPAD_CENTER); 
			return true; 
		case KeyEvent.KEYCODE_DPAD_DOWN: 
			Log.v("hey","pressed " + KeyEvent.KEYCODE_DPAD_DOWN); 
			return true; 
		case KeyEvent.KEYCODE_DPAD_LEFT: 
			prevPage();
			//	                Log.v("hey","pressed " + KeyEvent.KEYCODE_DPAD_LEFT); 
			return true; 
		case KeyEvent.KEYCODE_DPAD_RIGHT:
			nextPage();
			//	                Log.v("hey","pressed " + KeyEvent.KEYCODE_DPAD_RIGHT); 
			return true; 
		case KeyEvent.KEYCODE_DPAD_UP: 
			aerialSwitch = !aerialSwitch;
			if (aerialSwitch) {
//				contents.setInAnimation(Animations.inFromTopAnimation());
//				contents.setOutAnimation(Animations.outToBottomAnimation());
				contents.setDisplayedChild(contents.getChildCount() - 1);
			} else {
//				contents.setInAnimation(Animations.inFromBottomAnimation());
//				contents.setOutAnimation(Animations.outToTopAnimation());
				contents.setDisplayedChild(currentPageIndex);
				pages.get(currentPageIndex).setVisited(true);
			}
			contents.getChildAt(currentPageIndex).setBackgroundDrawable(setImage(pages.get(currentPageIndex).getPageImage()));
			reconstruct();
			//	                Log.v("hey","pressed " + KeyEvent.KEYCODE_DPAD_UP); 
			return true; 
		} 
		return super.onKeyDown(keyCode, event); 
	} 
}
