package edu.byuh.cis.jarnaby;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import android.graphics.Rect;
import android.graphics.drawable.BitmapDrawable;
import android.util.Log;

public class StoryParser extends DefaultHandler {

	public static final String ROOT_NODE = "jarnaby";
	public static final String VERSION = "version";
	public static final String STORY = "story";
	public static final String ABOUT = "about";
	public static final String AUTHOR = "author";
	public static final String ARTWORK = "artwork";
	public static final String MUSIC = "music";
	public static final String VOICE = "voice";
	public static final String SUMMARY = "summary";
	public static final String TITLE = "title";
	public static final String BKGD_AUDIO = "background_audio";
	public static final String IMAGE = "image";
	public static final String ID = "id";
	public static final String FILE = "file";
	public static final String PAGE = "page";
	public static final String IMAGE_ID = "image_id";
	public static final String AUDIO_FILE = "audio_file";
	public static final String TEXT = "text";
	public static final String LINK = "link";
	public static final String TRACKREF = "pathref";
	public static final String PAGEREF = "pageref";
	public static final String RECT = "rect";
	public static final String X = "x";
	public static final String Y = "y";
	public static final String WIDTH = "width";
	public static final String HEIGHT = "height";
	public static final String PATH = "path";
	public static final String NAME = "name";
	public static final String DEFAULT = "default";
	public static final String BKGD_TEXTURE = "graph_background";

	private String storyTitle;
	private String titleFile;
	private String bkgdMusic;
	private Map<String, String> images = new HashMap<String, String>();
	private Map<String, String> credits = new HashMap<String, String>();
	private Map<Integer, Page> pages = new HashMap<Integer, Page>();
	private Map<Integer, Path> paths = new HashMap<Integer, Path>();
	private List<TempLink> tmpLinks = new ArrayList<TempLink>();
	private Page currentPage;
	private TempLink currentLink;
	private Path currentPath;
	private Path defaultPath;
	private State state = State.DEFAULT;
	private BitmapDrawable graphBackground;
	private int screenWidth, screenHeight;

	private enum State {
		EXPECTING_TITLE,
		EXPECTING_TEXT,
		DEFAULT
	}

	private class TempLink {
		Page fromPage;
		int toPageID, toPath;
		Rect rect;
		String text;
	}

	public StoryParser(int deviceWidth, int deviceHeight) {
		super();
		currentPage = null;
		currentLink = null;
		screenWidth = deviceWidth;
		screenHeight = deviceHeight;
	}

	@Override
	public void startElement (String junk1, String junk2, String tag, Attributes atts) {
		if ("".equals (junk1)) {
//			if (tag.equals(ROOT_NODE)) {
//				float version = Float.parseFloat(atts.getValue(VERSION));
//			}
			
			if (tag.equals(ABOUT)) { 
				try { 
					String author = atts.getValue(AUTHOR);				
					String artwork = atts.getValue(ARTWORK);
					String music = atts.getValue(MUSIC);
					String voice = atts.getValue(VOICE);
					String summary = atts.getValue(SUMMARY);
					credits.put(AUTHOR, author);
					credits.put(ARTWORK, artwork);
					credits.put(MUSIC, music);
					credits.put(VOICE, voice);
					credits.put(SUMMARY, summary);
//					Log.d(Main.TAG, "filling Map from xml values such as... "+ author); 
				} catch(Exception e){ 
					//Log.i(Main.TAG, "Error: " + e.toString()); 
				}
			}
			
			if (tag.equals(TITLE)) {
				state = State.EXPECTING_TITLE;
				try { 				
					titleFile = Config.getImagePath() + atts.getValue(FILE);
				} catch(Exception e){ 
					//Log.i(Main.TAG, "Error: " + e.toString()); 
				}
			}
			
			if (tag.equals(BKGD_AUDIO)) { 
				try { 
					bkgdMusic = Config.getAudioPath() + atts.getValue(FILE);
//					Log.d(Main.TAG, "Parser PATH: " + bkgdMusic); 
				} catch(Exception e){ 
					//Log.i(Main.TAG, "Error: " + e.toString()); 
				}
			}
			
			if (tag.equals(IMAGE)) { 
				try { 
					String id = atts.getValue(ID);				
					String icon = Config.getImagePath() + atts.getValue(FILE);
					images.put(id, icon);
				} catch(Exception e){ 
					//Log.i(Main.TAG, "Error: " + e.toString()); 
				}
			}
			if (tag.equals(PAGE)) {
				int id = Integer.parseInt(atts.getValue(ID));
				String pid = atts.getValue(IMAGE_ID);
				String sound = Config.getAudioPath() + atts.getValue(AUDIO_FILE);
				Page p = new Page(id, pid, sound);
				pages.put(id, p);
				currentPage = p;
			}
			if (tag.equals(TEXT)) {
				if (currentPage != null) {
					state = State.EXPECTING_TEXT;
				}
			}
			if (tag.equals(LINK)) {
				int track = Integer.parseInt(atts.getValue(TRACKREF));
				int pageID = Integer.parseInt(atts.getValue(PAGEREF));
				String text = atts.getValue(TEXT);
				TempLink tmp = new TempLink();
				tmp.fromPage = currentPage;

				//Log.i(Main.TAG, text + "'s link is at: " + currentPage.getID()); 
				
				tmp.toPageID = pageID;
				tmp.toPath = track;
				tmp.text = text;
				tmpLinks.add(tmp);
				currentLink = tmp;
			}
			if (tag.equals(RECT)) {
				int x = (int)((Double.parseDouble(atts.getValue(X)))*screenWidth);
				int y = (int)((Double.parseDouble(atts.getValue(Y)))*screenHeight-screenHeight/10);
				int w = (int)((Double.parseDouble(atts.getValue(WIDTH)))*screenWidth);
				int h = (int)((Double.parseDouble(atts.getValue(HEIGHT)))*screenHeight);
				currentLink.rect = new Rect(x,y,w,h);
			}
			if (tag.equals(PATH)) {
				int id = Integer.parseInt(atts.getValue(ID));
				String name = atts.getValue(NAME);
				boolean defalt = Boolean.parseBoolean(atts.getValue(DEFAULT));
				currentPath = new Path(name, id);
				if (defalt) currentPath.makeDefault();
				paths.put(id, currentPath);
				if (defalt) {
					defaultPath = currentPath;
				}
			}
			if (tag.equals(PAGEREF)) {
				int pid = Integer.parseInt(atts.getValue(ID));
				currentPath.placeholders.add(pid);
			}

		} else{
			//Log.i(Main.TAG, "Start element: {" + junk1 + "}" + junk2);
		}
	}

	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		String foo = new String(ch, start, length).trim();
		switch (state) {
		case EXPECTING_TITLE:
			storyTitle = foo;
			//Log.i(Main.TAG, "The title of our story is "  + storyTitle);
			state = State.DEFAULT;
			break;
		case EXPECTING_TEXT:
			if (currentPage != null) {
				currentPage.addText(foo);
				state = State.DEFAULT;
			}		
			break;
		}
	}

	@Override
	public void endElement (String junk1, String junk2, String tag)
	{
		if ("".equals (junk1)) {
			if (tag.equals(TITLE)) {
				state = State.DEFAULT;
			}
			if (tag.equals(PAGE)) {
				state = State.DEFAULT;
				currentPage = null;
			}
			if (tag.equals(LINK)) {
				state = State.DEFAULT;
				currentLink = null;
			}
			if (tag.equals(TEXT)) {
				state = State.DEFAULT;
			}

		}	
		else {
			//Log.i(Main.TAG, "End element:   {" + junk1 + "}" + junk2);
		}
	}

	@Override
	public void endDocument() {

		//update link references to pages that maybe weren't there when
		//we parsed the first time.
		for (TempLink tl : tmpLinks) {
			Link ln = Link.createPartialLink(paths.get(tl.toPath), tl.toPageID);
			ln.rect = tl.rect;
			tl.fromPage.addLink(ln);
		}

		//update the paths, in case of page references we haven't parsed yet
		for (Path path : paths.values()) {
			for (int j=0; j<path.placeholders.size(); j++) {
				//path.addPage(pages.get(path.placeholders.get(j)));
				int id = path.placeholders.get(j);
				path.addPage(id, pages.get(id));
			}
		}

		//update the links so they point to the index of the page relative to
		//its path, instead of using absolute page ID numbers.
		for (Path path : paths.values()) {
			path.resolveLinks();
		}
	}

	public Map<Integer, Path> getPaths() {
		return paths;
	}

	public Path getDefaultPath() {
		return defaultPath;
	}

	public Map<Integer, Page> getPages() {
		return pages;
	}

	public BitmapDrawable getGraphBackground() {
		return graphBackground;
	}

	//added
	public Map<String, String> getImages(){
		return images;
	}
	
	public Map<String, String> getCredits(){
		return credits;
	}
	
	public String getTitle() {
		return storyTitle;
	}
	
	public String getMusic(){
		return bkgdMusic;
	}
	
	public String getTitleFile(){
		return titleFile;
	}

}
